@@ -9,7 +9,7 @@ use strict;
 use warnings FATAL => 'all';
 
 # Set the version at compile time, since some other modules borrow it.
-our $VERSION = "0.44"; # UR $VERSION
+our $VERSION = "0.45"; # UR $VERSION
 
 # Ensure we get detailed errors while starting up.
 # This is disabled at the bottom of the module.
@@ -289,6 +289,7 @@ UR::Object::Type->define(
         # UR::Namespaces are singletons referenced through their name
         namespace_meta                  => { is => 'UR::Namespace', id_by => 'namespace' },
         is                              => { is => 'ARRAY', is_mutable => 0, doc => 'List of the parent class names' },
+        roles                           => { is => 'ARRAY', is_mutable => 0, is_optional => 1, doc => 'List of the roles consumed by this class' },
 
         # linking to the direct parents, and the complete ancestry
         parent_class_metas              => { is => 'UR::Object::Type', id_by => 'is',
@@ -428,6 +429,7 @@ UR::Object::Type->define(
 require UR::Context;
 UR::Object::Type->initialize_bootstrap_classes;
 
+require UR::Role;
 require Command;
 
 $UR::initialized = 1;
@@ -473,7 +475,7 @@ UR - rich declarative transactional objects
 
 =head1 VERSION
 
-This document describes UR version 0.44
+This document describes UR version 0.45
 
 =head1 SYNOPSIS
 
@@ -745,8 +747,8 @@ XML::Simple
 
 =head1 AUTHORS
 
-UR was built by the software development team at the McDonnell Genome
-Institute at Washington University School of Medicine (Richard K. Wilson, PI).
+UR was built by the software development team at the McDonnell Genome Institute
+at the Washington University School of Medicine (Richard K. Wilson, PI).
 
 Incarnations of it run laboratory automation and analysis systems
 for high-throughput genomics.
@@ -794,7 +796,7 @@ for high-throughput genomics.
 
 =head1 LICENCE AND COPYRIGHT
 
-Copyright (C) 2002-2015 Washington University in St. Louis, MO.
+Copyright (C) 2002-2016 Washington University in St. Louis, MO.
 
 This sofware is licensed under the same terms as Perl itself.
 See the LICENSE file in this distribution.
