@@ -3,7 +3,7 @@ use strict;
 use warnings;
 
 require UR;
-our $VERSION = "0.44"; # UR $VERSION;
+our $VERSION = "0.45"; # UR $VERSION;
 use Sys::Hostname;
 
 {
@@ -56,6 +56,12 @@ sub table_and_column_names_are_upper_case { 0; }
 # join within irs own code
 sub does_support_joins { 0; }
 
+# Many data sources do not support limit and offset.
+sub does_support_limit_offset {
+    #my($self, $query_plan) = @_;
+    0
+}
+
 # Most datasources do not support recursive queries
 # Oracle and Postgres do, but in different ways
 # For data sources without support, it'll have to do multiple queries
@@ -505,7 +511,7 @@ sub _first_class_in_inheritance_with_a_table {
     my $found = "";
     for ($class_object, $class_object->ancestry_class_metas)
     {                
-        if ($_->table_name)
+        if ($_->has_direct_table)
         {
             $found = $_->class_name;
             last;
