@@ -3,7 +3,7 @@ use warnings;
 use strict;
 
 require UR;
-our $VERSION = "0.44"; # UR $VERSION;
+our $VERSION = "0.45"; # UR $VERSION;
 
 use Sys::Hostname;
 use Cwd;
@@ -36,7 +36,9 @@ sub data_source {
     my $ds = $self->data_source_id(@_);
     
     return undef unless $ds;
-    my $obj = UR::DataSource->get($ds) || $ds->get();
+    local $@;
+    my $obj = eval { UR::DataSource->get($ds) || $ds->get() };
+
     return $obj;
 }
 
@@ -207,11 +209,16 @@ sub resolve_property_aliases {
             # there was a problem resolving the chain of properties
             # This happens in the case of an object accessor (is => 'Some::Class') without an id_by
             my @split_names = split(/\./,$property_name);
+            my $name_count = @split_names;
             my $prop_meta = $self->property_meta_for_name(shift @split_names);
             return unless $prop_meta;
             my $foreign_class = $prop_meta->data_type && eval { $prop_meta->data_type->__meta__};
             return unless $foreign_class;
             @property_names = ( $prop_meta->alias_for, $foreign_class->resolve_property_aliases(join('.', @split_names)));
+            unless (@property_names >= $name_count) {
+                Carp::croak("Some parts from property '$property_name' of class ".$self->class_name
+                            . " didn't resolve");
+            }
         }
         $self->{'_resolve_property_aliases'}->{$property_name} = join('.', @property_names);
     }
@@ -583,9 +590,6 @@ sub id_property_sorter {
 }
 
 sub sorter {
-    #TODO: make this take +/- indications of ascending/descending
-    #TODO: make it into a closure for speed
-    #TODO: there are possibilities of it sorting different than a DB on mixed numbers and alpha data
     my ($self,@properties) = @_;
     push @properties, $self->id_property_names;
     my $key = join("__",@properties);
@@ -601,19 +605,10 @@ sub sorter {
                 push @is_descending, 0;
             }
 
-            my $class_meta;
-            if ($self->isa("UR::Object::Set::Type")) {
-                # If we're a set, we want to examine the property of our members.
-                my $subject_class = $self->class_name;
-                $subject_class =~ s/::Set$//g;
-                $class_meta = $subject_class->__meta__;#->property($property);
-            } else {
-                $class_meta = $self;
-            }
-
-            my ($pmeta,@extra) = $class_meta->_concrete_property_meta_for_class_and_name($property);
+            my ($pmeta,@extra) = $self->_concrete_property_meta_for_class_and_name($property);
             if(@extra) {
-                $pmeta = $class_meta->property($property); #a composite property (typically ID)
+                # maybe a composite property (typically ID), or a chained property (prop.other_prop)
+                $pmeta = $self->property_meta_for_name($property);
             }
 
             if ($pmeta) {
@@ -629,7 +624,7 @@ sub sorter {
             }
         }
 
-        no warnings;   # don't print a warning about undef values ...alow them to be treated as 0 or '' 
+        no warnings 'uninitialized';
         $sorter = $self->{_sorter}{$key} ||= sub($$) {
 
             for (my $n = 0; $n < @properties; $n++) {
@@ -861,6 +856,7 @@ sub generate_support_class_for_extension {
         delete $class_params{id_generator};
         delete $class_params{id};
         delete $class_params{is};
+        delete $class_params{roles};
 
         my $attributes_have = UR::Util::deep_copy($subject_class_obj->{attributes_have});
         my $class_props = UR::Util::deep_copy($subject_class_obj->{has});    
@@ -900,13 +896,24 @@ sub has_table {
     for my $class_name (@parent_classes) {
         next if $class_name eq "UR::Object";
         my $class_obj = UR::Object::Type->get(class_name => $class_name);
-        if ($class_obj->table_name) {
+        if ($class_obj->has_direct_table) {
             return 1;
         }
     }
     return;
 }
 
+sub has_direct_table {
+    my $self = shift;
+    return 1 if $self->table_name;
+
+    if ($self->data_source_id and $self->data_source_id->isa('UR::DataSource::Default')) {
+        my $load_function_name = join('::', $self->class_name, '__load__');
+        return 1 if exists &$load_function_name;
+    }
+    return;
+}
+
 sub most_specific_subclass_with_table {
     my $self = shift;
 
@@ -914,7 +921,7 @@ sub most_specific_subclass_with_table {
 
     foreach my $class_name ( $self->class_name->inheritance ) {
         my $class_obj = UR::Object::Type->get(class_name => $class_name);
-        return $class_name if ($class_obj && $class_obj->table_name);
+        return $class_name if ($class_obj and $class_obj->has_direct_table);
     }
     return;
 }
@@ -925,7 +932,7 @@ sub most_general_subclass_with_table {
     my @subclass_list = reverse ( $self->class_name, $self->class_name->inheritance );
     foreach my $class_name ( $self->inheritance ) {
         my $class_obj = UR::Object::Type->get(class_name => $class_name);
-        return $class_name if ($class_obj && $class_obj->table_name);
+        return $class_name if ($class_obj && $class_obj->has_direct_table);
     }
     return;
 }
@@ -1010,8 +1017,9 @@ sub _load {
         # "Can't locate UR/Object/Type/Ghost.pm in @INC" error.
         # We want to fall through "in the right circumstances".
         (my $module_path = $class_name . '.pm') =~ s/::/\//g;
-        Carp::croak("Error while autoloading with 'use $class_name': $exception") unless ($exception =~ /Can't locate $module_path in \@INC/);
-        # FIXME: I think other conditions here will result in silent errors.
+        unless ($exception =~ /Can't locate $module_path in \@INC/) {
+            die "Error while autoloading with 'use $class_name': $exception";
+        }
     }
 
     # Parse the specified class name to check for a suffix.
@@ -1039,7 +1047,15 @@ sub _load {
         # which would fire recursively for three extensions of
         # Acme::Equipment.
         my $full_base_class_name = $prefix . ($base ? "::" . $base : "");
-        my $base_class_obj = eval { $full_base_class_name->__meta__ };
+        my $base_class_obj;
+        my $exception = do {
+            local $@;
+            $base_class_obj = eval { $full_base_class_name->__meta__ };
+            $@;
+        };
+        if ($exception && $exception =~ m/^Error while autoloading/) {
+            die $exception;
+        }
 
         if ($base_class_obj)
         {
