@@ -8,7 +8,7 @@ require UR;
 use List::MoreUtils qw(first_index);
 use Scalar::Util qw(looks_like_number);
 
-our $VERSION = "0.44"; # UR $VERSION;
+our $VERSION = "0.45"; # UR $VERSION;
 
 our %meta_classes;
 our $bootstrapping = 1;
@@ -109,7 +109,7 @@ sub resolve_class_description_perl {
 
     # These property names are either written in other places in this sub, or shouldn't be written out
     my %addl_property_names = map { $_ => 1 } $self->__meta__->all_property_type_names;
-    my @specified = qw/is class_name table_name id_by er_role is_abstract generated data_source_id schema_name doc namespace id first_sub_classification_method_name property_metas pproperty_names id_property_metas meta_class_name id_generator valid_signals/;
+    my @specified = qw/is class_name table_name id_by er_role is_abstract generated data_source_id schema_name doc namespace id first_sub_classification_method_name property_metas pproperty_names id_property_metas meta_class_name id_generator valid_signals roles/;
     delete @addl_property_names{@specified};
     for my $property_name (sort keys %addl_property_names) {
         my $property_obj = $self->__meta__->property_meta_for_name($property_name);
@@ -225,6 +225,10 @@ sub resolve_class_description_perl {
         $perl .= "    id_generator => '$id_generator',\n" if ($print_id_generator);
     }
 
+    if ($self->roles and @{ $self->roles }) {
+        $perl .= "    roles => " . pprint_arrayref($self->roles) . ",\n";
+    }
+
     if (my $valid_signals = $self->valid_signals) {
         if (ref($valid_signals) ne 'ARRAY') {
             Carp::croak("The 'valid_signals' metadata for class $class_name must be an arrayref, got: ".Data::Dumper::Dumper($valid_signals));
